@@ -10,8 +10,8 @@ use crate::{
 #[async_trait(?Send)]
 pub trait ConstructedDependencySetStrategy {
     async fn set_transient<TType: Constructor>(&self) -> AddDependencyResult<()>;
-    async fn set_singleton<TType: Constructor>(&self) -> AddDependencyResult<()>;
-    async fn set_scoped<TType: Constructor>(&self) -> AddDependencyResult<()>;
+    async fn set_singleton<TType: Sync + Send + Constructor>(&self) -> AddDependencyResult<()>;
+    async fn set_scoped<TType: Sync + Send + Constructor>(&self) -> AddDependencyResult<()>;
 }
 
 #[async_trait(?Send)]
@@ -20,11 +20,11 @@ impl ConstructedDependencySetStrategy for DependencyContext {
         self.add_transient::<TType>(Box::new(BaseConstructor::new::<TType>())).await
     }
 
-    async fn set_singleton<TType: Constructor>(&self) -> AddDependencyResult<()> {
+    async fn set_singleton<TType: Sync + Send + Constructor>(&self) -> AddDependencyResult<()> {
         self.add_singleton::<TType>(Box::new(BaseConstructor::new::<TType>())).await
     }
 
-    async fn set_scoped<TType: Constructor>(&self) -> AddDependencyResult<()> {
+    async fn set_scoped<TType: Sync + Send + Constructor>(&self) -> AddDependencyResult<()> {
         self.add_scoped::<TType>(Box::new(BaseConstructor::new::<TType>())).await
     }
 }
\ No newline at end of file
